# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-15 22:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
from django.utils.text import slugify


def derridawork_slugs(apps, schema_editor):
    DerridaWork = apps.get_model('books', 'DerridaWork')
    for work in DerridaWork.objects.all():
        work.slug = slugify(work.short_title)
        work.save()


def truncate_to_int(apps, schema_editor):
    '''
    Make derridawork_pages safe for migration to convert to int by removing
    page ranges.
    '''
    Reference = apps.get_model('books', 'Reference')
    for ref in Reference.objects.all():
        try:
            # see if it can be made into an int and therefore is safe for the
            # AlterField statement
            int(ref.derridawork_page)
        except ValueError:
            # if not choose the leftmost page of the reference and save
            ref.derridawork_page = ref.derridawork_page.split('-')[0]
            ref.save()


class Migration(migrations.Migration):

    dependencies = [
        ('books', '0003_add_foreignkey_reference_canvas_intervention'),
    ]

    operations = [
        migrations.CreateModel(
            name='DerridaWorkSection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('order', models.PositiveIntegerField(verbose_name='Order')),
                ('start_page', models.IntegerField(blank=True, null=True,
                    help_text='Sections with no pages will be treated as headers.')),
                ('end_page', models.IntegerField(blank=True, null=True)),
            ]
        ),
        migrations.AddField(
            model_name='derridawork',
            name='slug',
            field=models.SlugField(default='', help_text='slug for use in URLs (changing after creation will break URLs)'),
            preserve_default=False,
        ),
        migrations.RunPython(
            code=truncate_to_int,
            reverse_code=migrations.RunPython.noop,
        ),
        migrations.AlterField(
            model_name='reference',
            name='derridawork_page',
            field=models.IntegerField(),
        ),
        migrations.AddField(
            model_name='derridaworksection',
            name='derridawork',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='books.DerridaWork'),
        ),
        migrations.RunPython(
            code=derridawork_slugs,
            reverse_code=migrations.RunPython.noop,
        ),

    ]
