# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-10 18:11
from __future__ import unicode_literals

from django.contrib.auth.management import create_permissions
from django.db import migrations


def add_permissions(apps, schema_editor):
    # ensure permissions have been updated
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=3)
        app_config.models_module = None

    Group = apps.get_model('auth', 'Group')
    Permission = apps.get_model('auth', 'Permission')
    try:
        data_editor_group = Group.objects.get(name='Data Editors')
        new_permissions = [
            # permissions on footnote models
            'add_bibliography', 'change_bibliography', 'delete_bibliography',
            'add_footnote', 'change_footnote', 'delete_footnote',
            'add_sourcetype', 'change_sourcetype', 'delete_sourcetype',
            # permissions to view iiif content
            'view_manifest', 'view_canvas',
            # permissions for intervention/annotation
            'add_intervention', 'change_intervention', 'delete_intervention',
            'view_intervention',
            # annotation api views test against base annotation permissions
            'add_annotation', 'change_annotation', 'delete_annotation',
            'view_annotation',
        ]

        # NOTE: finding one by one rather than using in_list so that if a
        # permission is not found we get an error
        for codename in new_permissions:
            try:
                perm = Permission.objects.get(codename=codename)
                data_editor_group.permissions.add(perm)
            except:
                pass
    except Group.DoesNotExist:
        # if group has somehow been removed, then do nothing
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0001_data_editor_group'),
        ('footnotes', '0001_initial'),
        ('djiffy', '0002_view_permissions'),
        ('interventions', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(add_permissions, reverse_code=migrations.RunPython.noop),
    ]
